using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;
using Object = UnityEngine.Object;

public class Storyboard : MVRScript
{
    private JSONStorableStringChooser actionsChooser;

    private VerticalLayoutGroup _container;

    private GameObject _child;

    private GameObject _lineObject;

    private Canvas _canvas;

    private Dictionary<string, UIDynamicButton> buttons = new Dictionary<string, UIDynamicButton>();
    public override void Init()
    {
        try
        {
            InitOptionsUI();
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(Storyboard)}.{nameof(Init)}: {e}");
        }
    }

    private void InitOptionsUI()
    {
        Utils.OnInitUI(CreateUIElement);
        Utils.SetupButton(this, "Test Execute Action", OnExecuteActionButtonClicked, false).buttonColor = new Color(0.45f, 1f, 0.45f);

        var atom = this.containingAtom;

        _canvas = atom.GetComponentsInChildren<Canvas>().FirstOrDefault(c => c.gameObject.name == "Sign");
        if (_canvas == null) throw new NullReferenceException($"Expected Canvas called 'Sign', but found canvases: [{string.Join(",", atom.GetComponentsInChildren<Canvas>().Select(c => c.gameObject.name).ToArray())}]");

        _child = new GameObject("Simple Sign UI");
        _child.transform.SetParent(_canvas.transform, false);

        var rect = _child.AddComponent<RectTransform>();
        rect.anchorMin = Vector2.zero;
        rect.anchorMax = Vector2.one;
        rect.offsetMin = new Vector2(0f, 0f);
        rect.offsetMax = new Vector2(1500f, 500f);

        var bg = _child.AddComponent<GradientImage>();
        var color = new Color(0.30f, 0.36f, 0.52f);
        bg.top = Color.Lerp(color, Color.black, 0.18f);
        bg.bottom = Color.Lerp(color, Color.white, 0.03f);

        // var button = CreateUIButtonInCanvas("Stand", rect, 100, 100);
        // button.button.onClick.AddListener(OnButtonInCanvasClicked);

        // var button2 = CreateUIButtonInCanvas("Sit", rect);
        // button2.button.onClick.AddListener(OnButtonInCanvasClicked);

        // var button3 = CreateUIButtonInCanvas("Swim", rect, 1000, 500);

        // DrawLineBetweenButtons(button3, button);
        // DrawLineBetweenButtons(button, button2);




        // ActionItem sit = new ActionItem("sit", "", "", "", "", "", "", "", false, "", null);
        // ActionItem drink = new ActionItem("drink", "", "", "", "", "", "", "", false, "", null);
        // ActionItem swim = new ActionItem("swim", "", "", "", "", "", "", "", false, "", null);
        // ActionItem standUp = new ActionItem("standUp", "", "", "", "", "", "", "", false, "", null);
        // ActionItem stand = new ActionItem("stand", "", "", "", "", "", "", "", false, "", null);
        // ActionItem smile = new ActionItem("smile", "", "", "", "", "", "", "", false, "", null);
        // ActionItem wearPJ = new ActionItem("wearPJ", "", "", "", "", "", "", "", false, "", null);
        // ActionItem wearClothes = new ActionItem("wearClothes", "", "", "", "", "", "", "", false, "", null);




        // sit.LinkedActions.AddRange(new string[] { drink.Name, swim.Name, standUp.Name, smile.Name, wearPJ.Name, wearClothes.Name });
        // drink.LinkedActions.AddRange(new string[] { wearPJ.Name, wearClothes.Name });
        // swim.LinkedActions.AddRange(new string[] { smile.Name, wearClothes.Name });
        // standUp.LinkedActions.AddRange(new string[] { swim.Name, stand.Name, smile.Name, wearPJ.Name, wearClothes.Name });
        // stand.LinkedActions.AddRange(new string[] { sit.Name, swim.Name, standUp.Name, smile.Name, wearPJ.Name });

        // CreateButtons(new List<ActionItem> { sit, drink, swim, stand, standUp, smile, wearPJ, wearClothes }, rect);
    }

    private void CreateButtons(List<ActionItem> actionItems, RectTransform rect)
    {
        // Wstępne pozycjonowanie przycisków
        float horizontalSpacing = 200.0f; // Ustaw odstępy poziome
        float verticalSpacing = 100.0f; // Ustaw odstępy pionowe
        Vector2 startPosition = new Vector2(-horizontalSpacing * (actionItems.Count / 2), 0); // Centrum ekranu

        for (int i = 0; i < actionItems.Count; i++)
        {
            // Oblicz pozycję dla każdego przycisku
            Vector2 position = new Vector2(startPosition.x + (horizontalSpacing * i), startPosition.y + (verticalSpacing * (i % 2)));

            // Utwórz przyciski dla każdego ActionItem
            UIDynamicButton button = CreateUIButtonInCanvas(actionItems[i].Name, rect, position.x, position.y);

            // Zapisz RectTransform przycisku w słowniku
            buttons.Add(actionItems[i].Name, button);
        }
    }

    private void OnButtonInCanvasClicked()
    {
        getPluginByType(GetAtomById("Person"), "ActionListManager").GetStringJSONParam("TriggerSelectedActionChange").val = "Shy pose";
    }

    public UIDynamicButton CreateUIButtonInCanvas(string label, RectTransform parent, float x = 0, float y = 0)
    {
        var transform = Object.Instantiate(this.manager.configurableButtonPrefab.transform, parent, false) as RectTransform;
        if (transform == null) throw new NullReferenceException("Could not instantiate configurableButtonPrefab");
        transform.gameObject.SetActive(true);
        transform.sizeDelta = new Vector2(-2300, transform.sizeDelta.y);
        transform.anchoredPosition = new Vector2(-1150 + x, -1675 + y);
        // rectTransform.anchoredPosition = new Vector2(-1000, -100);

        var ui = transform.GetComponent<UIDynamicButton>();
        if (ui == null) throw new NullReferenceException("Could not find a UIDynamicButton component");
        ui.label = label;

        return ui;
    }

    public void AdjustButtonWidth(GameObject button, float newWidth)
    {
        RectTransform rectTransform = button.GetComponent<RectTransform>();
        if (rectTransform != null)
        {
            // Ustawienie nowej szerokości
            rectTransform.sizeDelta = new Vector2(newWidth, rectTransform.sizeDelta.y);
        }
        else
        {
            Debug.LogError("RectTransform component not found on the button");
        }
    }
    public void DrawLineBetweenButtons(UIDynamicButton button1, UIDynamicButton button2)
    {
        var anchoredPosition1 = button1.GetComponent<RectTransform>().anchoredPosition;
        var anchoredPosition2 = button2.GetComponent<RectTransform>().anchoredPosition;

        // Obliczenie pozycji startowej i końcowej
        Vector2 startPos = new Vector2(anchoredPosition1.x, anchoredPosition1.y + 850);
        Vector2 endPos = new Vector2(anchoredPosition2.x, anchoredPosition2.y + 850);

        // Rysowanie linii
        DrawUILine(_child.GetComponent<RectTransform>(), startPos, endPos, 2f, Color.white);
    }

    public void DrawUILine(RectTransform parent, Vector2 startPos, Vector2 endPos, float width, Color color)
    {
        GameObject line = new GameObject("UILine");
        line.transform.SetParent(parent, false);

        Image lineImage = line.AddComponent<Image>();
        lineImage.color = color;

        RectTransform rectTransform = line.GetComponent<RectTransform>();
        Vector2 direction = (endPos - startPos).normalized;
        float distance = Vector2.Distance(startPos, endPos);

        rectTransform.sizeDelta = new Vector2(distance, width);
        rectTransform.anchorMin = rectTransform.anchorMax = Vector2.zero;
        rectTransform.pivot = new Vector2(0, 0.5f);
        rectTransform.localPosition = startPos;
        rectTransform.localRotation = Quaternion.Euler(0, 0, Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg);
    }

    private void OnExecuteActionButtonClicked()
    {

    }

    public void Update()
    {
        try
        {
        }
        catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
    }

    public static JSONStorable getPluginByType(Atom atom, string pluginType)
    {
        foreach (string pluginId in atom.GetStorableIDs())
        {
            if (pluginId.Contains(pluginType))
            {
                return atom.GetStorableByID(pluginId);
            }
        }

        throw new NullReferenceException("Cannot find plugin of type " + pluginType + " on atom " + atom.uid);
    }

    void OnDestroy()
    {
        if (_canvas != null)
        {
            Object.Destroy(_child);
            Object.Destroy(_lineObject);
        }
    }

    public override bool ShouldIgnore() => false;
}
