using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using static JSONStorableString;
public class UIAssistProvider
{
    JSONStorableString lastButtonPressValueJSS;
    JSONStorableAction forceButtonListDataRefreshJSA = new JSONStorableAction("forceButtonListDataRefresh", null);
    SetStringCallback actionExecutorHandler;
    private List<string> actionList = new List<string>();

    public UIAssistProvider(SetStringCallback actionExecuteHandler)
    {
        this.actionExecutorHandler = actionExecuteHandler;
        lastButtonPressValueJSS = new JSONStorableString("lastButtonPressValue", "", OnButtonPress);
    }

    private void OnButtonPress(string buttonName)
    {
        lastButtonPressValueJSS.valNoCallback = "";
        actionExecutorHandler.Invoke(buttonName);
    }

    public void ActionListChanged(List<string> actionList)
    {
        if (!this.actionList.SequenceEqual(actionList))
        {
            this.actionList = actionList;
            RequestRefresh();
        }
    }

    public void ClearUserActions()
    {
        actionList.Clear();
        RequestRefresh();
    }

    public void ChangeName(string oldName, string newName)
    {
        if (actionList.IndexOf(oldName) != -1)
        {
            actionList[actionList.IndexOf(oldName)] = newName;
            RequestRefresh();
        }
    }

    public void RemoveAction(string actionName)
    {
        if (actionList.IndexOf(actionName) != -1)
        {
            actionList.Remove(actionName);
            RequestRefresh();
        }
    }

    // This method will be called by the UIAssist plugin on any registered plugins. This method needs to populate the 'buttonListData' dictionary with specific values. The dictionary keys are pre-determined and the dictionary values will be readible by UIAssist. UIAssist will call the method again, if the JSONStorableAction forceButtonListDataRefreshJA is invoked.
    public void OnUIAButtonListDataRequested(Dictionary<string, object> buttonListData)
    {
        // This is included for future proofing, in case there is a need to radically change the integration spec in the future
        buttonListData.Add("IntegrationVersion", 1);

        // This identifies this source plugin, in case mutiple plugins attempt to register with UIAssist.
        buttonListData.Add("ButtonListProviderName", "ALM");

        // This is the list of button names to be passed to UIAssist. Each list entry will be presented as a button in UIAssist.
        buttonListData.Add("ButtonNameList", actionList);

        // This is the JSONStorable String that UIAssist will modify when one of the buttons is pressed. A callback should be set on this JSONStorableString to respond to when one of the buttons is pressed in UIAssist. The value of the JSONStorableString will allow this plugin to determine which button has been pressed in UIAssist.
        buttonListData.Add("ButtonPressCallbackJSS", lastButtonPressValueJSS);

        // This is a JSONStorableAction that UIAssist will populate with a callback. When the action is invoked (from this plugin), then UIAssist will call the OnUIAButtonListDateRequested method again to obtain an updated version of the list of buttons.
        buttonListData.Add("ForceButtonListDataRefresh", forceButtonListDataRefreshJSA);

        // This is an optional Dictionary of colors to assign to each button. The dictionary key should be the same name as the button name specified in the buttonNameList. Each entry is optional, so you only set the buttons that you want to change from the default grey color
        var buttonColors = new Dictionary<string, Color>();
        foreach (var actionName in actionList)
        {
            buttonColors.Add(actionName, Colors.BUTTON_USER_ACTION);
        }
        buttonListData.Add("ButtonColorsByName", buttonColors);

        buttonListData.Add("ButtonNamePin1", actionList[0]);
        // buttonListData.Add("GazeOpenCallback", new JSONStorableAction("log", () => { SuperController.LogError("Gaze opened"); }));
        // buttonListData.Add("GazeCloseCallback", new JSONStorableAction("log2", () => { SuperController.LogError("Gaze closed"); }));
    }

    private void RequestRefresh()
    {
        if (SuperController.singleton.isOVR && actionList.Count > 0)
        {
            OVRInput.SetControllerVibration(1f, 1f, OVRInput.Controller.LTouch);
            OVRInput.SetControllerVibration(1f, 1f, OVRInput.Controller.LTouch);

            OVRInput.SetControllerVibration(1f, 1f, OVRInput.Controller.RTouch);
            OVRInput.SetControllerVibration(1f, 1f, OVRInput.Controller.RTouch);
        }

        if (forceButtonListDataRefreshJSA.actionCallback != null)
        {
            forceButtonListDataRefreshJSA.actionCallback.Invoke();
        }
    }
}
