using System;
using UnityEngine;

public class Voxta
{
    public static JSONStorable GetPlugin(MVRScript script, bool optional = false)
    {
        return GetPluginByType(script.containingAtom, "Voxta", optional);
    }

    public static string GetState(MVRScript script)
    {
        return GetPlugin(script).GetStringChooserJSONParam("State").val;
    }

    public static string GetCharacterLastMessage(MVRScript script)
    {
        return GetPlugin(script).GetStringJSONParam("LastCharacterMessage").val;
    }

    public static string GetCurrentContext(MVRScript script)
    {
        return GetPlugin(script).GetStringJSONParam("Context").val;
    }

    public static string GetUserLastMessage(MVRScript script)
    {
        return GetPlugin(script).GetStringJSONParam("LastUserMessage").val;
    }

    public static string GetCurrentAction(MVRScript script)
    {
        return GetPlugin(script).GetStringJSONParam("CurrentAction").val;
    }

    public static void SetActionList(MVRScript script, string actionList)
    {
        GetPlugin(script).GetStringJSONParam("ActionsList").val = actionList;
    }

    public static void SetContext(MVRScript script, string context)
    {
        GetPlugin(script).GetStringJSONParam("Context").val = context;
    }

    public static void TriggerMessage(MVRScript script, string message)
    {
        GetPlugin(script).GetStringJSONParam("TriggerMessage").val = message;
    }

    public static void RevertLastSentMessage(MVRScript script)
    {
        GetPlugin(script).CallAction("Revert Last Sent Message");
    }

    public static void DeleteCurrentChat(MVRScript script)
    {
        GetPlugin(script).CallAction("Delete Current Chat");
    }

    public static void RequestCharacterSpeech(MVRScript script, string message)
    {
        GetPlugin(script).GetStringJSONParam("RequestCharacterSpeech").val = message;
    }

    public static void ToggleOn(MVRScript script)
    {
        GetPlugin(script).GetBoolJSONParam("Active").val = !IsActive(script);
    }

    public static bool IsActive(MVRScript script)
    {
        return GetPlugin(script).GetBoolJSONParam("Active").val;
    }

    public static JSONStorable GetPluginByType(Atom atom, string pluginType, bool optional = false)
    {
        foreach (string pluginId in atom.GetStorableIDs())
        {
            if (pluginId.Contains(pluginType))
            {
                return atom.GetStorableByID(pluginId);
            }
        }

        if (optional)
        {
            return null;
        }

        SuperController.LogError(StackTraceUtility.ExtractStackTrace());
        SuperController.LogError("Plugin with type " + pluginType + " has not been found on the atom " + atom.name);
        throw new NullReferenceException("Cannot find plugin of type " + pluginType + " on atom " + atom.uid);
    }
}
