using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;
using static MainUI;

public class BulkActionManagerUI : UIExtention
{
    private bool isOpen;
    private JSONStorableString searchField;
    private RectTransform actionsScroll;
    private UIDynamicButton closeButton;
    private GameObject assignToComponentBackground;
    private Dictionary<string, bool> groupExpanded = new Dictionary<string, bool>();
    private int groupBy = GroupByType.Group;

    private List<string> markedActionNames = new List<string>();
    private int markType;
    private static List<string> assignTolist = new List<string>
    {
        AssignToType.Group,
        AssignToType.Layer
    };
    private JSONStorableStringChooser assignTo;
    private JSONStorableStringChooser assignToName = new JSONStorableStringChooser("Actions", new List<string>(), "", "");

    public BulkActionManagerUI(ActionListManager script) : base(script)
    {
    }

    public override void Init()
    {
        isOpen = false;

        assignTo = new JSONStorableStringChooser("AssignTo", assignTolist, AssignToType.Group, "", ChangeAssignTo);

        Init(BulkActionManagerCoordinates.GET_BACKDROP_OFFSET_MIN(), BulkActionManagerCoordinates.GET_BACKDROP_OFFSET_MAX(), 1485, -1306);
        searchField = new JSONStorableString("Search", "", OnSearchChanged);
        actionsScroll = CreateScroll(backDrop.transform, new Vector2(0.01f, 0.16f), new Vector2(0.99f, 0.95f));

        new TextInputBuilder(script, uiElements, "Search:", searchField)
        .Parent(backDrop.transform).Location(new Vector2(5, -45)).Size(new Vector2(500, 40)).ConstantElement().Build();

        new ButtonBuilder(this, "X", () => { searchField.val = ""; })
        .Parent(backDrop.transform).Location(new Vector2(510, -55)).Size(new Vector2(-620, 50)).ConstantElement().Build();

        new ButtonBuilder(this, "+", () => FoldUnfoldAll(true))
        .Parent(backDrop.transform).Location(new Vector2(565, -55)).Size(new Vector2(-620, 50)).ConstantElement().Build();

        new ButtonBuilder(this, "-", () => FoldUnfoldAll(false))
        .Parent(backDrop.transform).Location(new Vector2(620, -55)).Size(new Vector2(-620, 50)).ConstantElement().Build();

        closeButton = UIHelpers.CreateCustomButton(script, "B>", BulkActionManagerCoordinates.GET_FOLD_LOCATION(), new Vector2(-1040, 690));
        closeButton.button.onClick.AddListener(() =>
        {
            backDrop.gameObject.SetActive(!backDrop.gameObject.activeSelf);
            closeButton.label = isOpen ? "B>" : "B<";
            if (!isOpen)
            {
                UI.newLinkedActionsUI.CloseBackdrop();
            }
            isOpen = !isOpen;
        });

        RefreshUI();

        backDrop.gameObject.SetActive(false);
    }

    public void CloseBackdrop()
    {
        isOpen = false;
        backDrop.gameObject.SetActive(false);
        closeButton.label = "B>";
    }

    private void ChangeAssignTo(string val)
    {
        RefreshUI();
    }

    private void FoldUnfoldAll(bool expanded)
    {
        foreach (var groupKey in groupExpanded.Keys.ToArray())
        {
            if (groupKey.StartsWith(groupBy + "-"))
            {
                groupExpanded[groupKey] = expanded;
            }
        }

        RefreshUI();
    }

    private void CopyRouting()
    {
        markType = MarkType.COPY_ROUTE;
        RefreshUI();
    }

    private void CopyFields()
    {
        markType = MarkType.COPY_FIELDS;
        RefreshUI();
    }

    private void AssignTo()
    {
        if (assignTo.val == AssignToType.Group)
        {
            markType = MarkType.ASSIGN_TO_GROUP;
        }
        else if (assignTo.val == AssignToType.Layer)
        {
            markType = MarkType.ASSIGN_TO_LAYER;
        }
        RefreshUI();
    }

    private void BulkRemove()
    {
        markType = MarkType.REMOVE;
        RefreshUI();
    }

    private List<IGrouping<string, ActionItem>> GroupActions()
    {
        if (groupBy == GroupByType.Group)
        {
            return script.actionsDictionary.Values
                   .GroupBy(a => a.Group)
                   .OrderBy(g => g.Key)
                   .ToList();
        }
        else if (groupBy == GroupByType.Layer)
        {
            return script.actionsDictionary.Values
                   .GroupBy(a => a.Layer)
                   .OrderBy(g => g.Key)
                   .ToList();
        }
        else
        {
            return script.actionsDictionary.Values
                   .GroupBy(a => a.Bundle)
                   .OrderBy(g => g.Key)
                   .ToList();
        }
    }

    private void CreateActionButton(ActionItem actionItem, RectTransform parent)
    {
        bool isSelected = markedActionNames.Contains(actionItem.Name);
        UnityAction actionButtonPressed = () => { SetChoosenAction(actionItem.Name); };
        if (isMarkMode())
        {
            if (!isSelected)
            {
                actionButtonPressed = () =>
                {
                    markedActionNames.Add(actionItem.Name);
                    RefreshUI();
                };
            }
            else
            {
                actionButtonPressed = () =>
                {
                    markedActionNames.Remove(actionItem.Name);
                    RefreshUI();
                };
            }
        }

        string additionalLabel = "";
        if (script.GetChoosenItem().LinkedActions.Contains(actionItem.Name))
            additionalLabel = "→";
        if (actionItem.LinkedActions.Contains(GetChosenAction()))
            additionalLabel = "←";
        if (script.GetChoosenItem().LinkedActions.Contains(actionItem.Name) && actionItem.LinkedActions.Contains(GetChosenAction()))
            additionalLabel = "↔";
        if (actionItem.Name == script.GetChoosenItem().Name)
            additionalLabel = "●";

        new CustomWidthButtonBuilder(this, actionItem.Name, actionButtonPressed).Parent(parent)
        .ButtonColor(isSelected ? Color.black : Colors.DefermineActionButtonColor(actionItem))
        .TextColor(isSelected ? Color.white : Color.black)
        .AdditionalLabel(additionalLabel)
        .Build();
    }

    private void OnSearchChanged(string val)
    {
        RefreshUI();
    }

    private void RefreshGroupExpansion(string groupName)
    {
        var groupExpansionKey = GetGroupExpansionKey(groupName);

        if (!groupExpanded.ContainsKey(groupExpansionKey))
        {
            if (groupBy == GroupByType.Bundle)
            {
                groupExpanded.Add(groupExpansionKey, false);
            }
            else
            {
                groupExpanded.Add(groupExpansionKey, true);
            }
        }
    }

    private void ToggleGroup(string groupName)
    {
        groupExpanded[GetGroupExpansionKey(groupName)] = !groupExpanded[GetGroupExpansionKey(groupName)];

        RefreshUI();
    }

    private string DetermineGroupName(string groupName)
    {
        if (groupBy == GroupByType.Group)
            return groupName != "" ? "Group: " + groupName : "Ungrouped";
        if (groupBy == GroupByType.Layer)
        {
            if (groupName != "" && script.layerDictionary[groupName].Global)
            {
                return "Global layer: " + groupName;
            }
            return groupName != "" ? "Layer: " + groupName : "No Layer";
        }
        if (groupBy == GroupByType.Bundle)
            return groupName != "" ? "Bundle: " + groupName : "Not From Bundle";

        return "unknown";
    }

    private void MarkUnMarkGroup(IGrouping<string, ActionItem> groupOnButton)
    {
        if (isMarkMode())
        {
            List<string> actionNamesInGroup = groupOnButton
                    .Select(actionItem => actionItem.Name)
                    .ToList();
            if (actionNamesInGroup.All(element => markedActionNames.Contains(element)))
            {
                markedActionNames.RemoveAll(element => actionNamesInGroup.Contains(element));
            }
            else
            {
                markedActionNames.AddRange(actionNamesInGroup);
            }

            RefreshUI();
        }
    }

    public override void RefreshUI()
    {
        RemoveAllUI();
        UnityEngine.Object.Destroy(assignToComponentBackground);

        closeButton.label = isOpen ? "B<" : "B>";

        new ThreeWayButtonBuilder(this, "Action Filter", () => { groupBy = GroupByType.Group; RefreshUI(); }, () => { groupBy = GroupByType.Layer; RefreshUI(); }, () => { groupBy = GroupByType.Bundle; RefreshUI(); }, groupBy).Parent(actionsScroll).Build();

        var spacer = script.CreateSpacer();
        spacer.transform.SetParent(actionsScroll);
        spacer.height = 10;
        uiElements.Add(spacer);

        foreach (var group in GroupActions())
        {
            string groupName = group.Key;
            RefreshGroupExpansion(groupName);

            UnityAction mainAction = () => MarkUnMarkGroup(group);

            UnityAction toggleAction = () => ToggleGroup(groupName);
            bool isGroupExpanded = !string.IsNullOrEmpty(searchField.val) || (groupExpanded.ContainsKey(GetGroupExpansionKey(groupName)) && groupExpanded[GetGroupExpansionKey(groupName)]);

            new TwinButtonBuilder(script, uiElements,
            DetermineGroupName(groupName),
            mainAction,
            isGroupExpanded ? "-" : "+",
            toggleAction).FirstWidth(0.9f).FirstStart(0)
            .ButtonColor(isWholeGroupSelected(group) ? Color.black : Colors.BUTTON_DEFAULT)
            .TextColor(isWholeGroupSelected(group) ? Color.white : Color.black)
            .LabelBold(true).Parent(actionsScroll).Build();

            if (isGroupExpanded)
            {
                IEnumerable<ActionItem> matchingActions;

                if (groupBy == GroupByType.Group)
                {
                    matchingActions = script.actionsDictionary.Values.Where(a => a.Group == groupName);
                }
                else if (groupBy == GroupByType.Layer)
                {
                    matchingActions = script.actionsDictionary.Values.Where(a => a.Layer == groupName);
                }
                else
                {
                    matchingActions = script.actionsDictionary.Values.Where(a => a.Bundle == groupName);
                }

                foreach (var actionItem in matchingActions)
                    if (groupName.ToLower().Contains(searchField.val.ToLower()) || actionItem.Name.ToLower().Contains(searchField.val.ToLower()))
                    {
                        if (!isMarkMode() || markType != MarkType.ROUTING || actionItem.Name != GetChosenAction())
                            CreateActionButton(actionItem, actionsScroll);
                    }

            }
        }

        DrawBottomButtons();
    }

    private bool isWholeGroupSelected(IGrouping<string, ActionItem> group)
    {
        if (isMarkMode())
        {
            List<string> actionNamesInGroup = group
                    .Select(actionItem => actionItem.Name)
                    .ToList();
            if (actionNamesInGroup.All(element => markedActionNames.Contains(element)))
            {
                return true;
            }
        }

        return false;
    }

    private void DrawBottomButtons()
    {
        if (!isMarkMode())
        {
            new ButtonBuilder(this, "Bulk Remove", BulkRemove)
            .Parent(backDrop.transform).Color(Colors.REMOVE_COLOR).Location(new Vector2(25, -1370)).Size(new Vector2(-470, 50)).Build();

            new ButtonBuilder(this, "Copy Routing", CopyRouting)
            .Parent(backDrop.transform).Location(new Vector2(235, -1370)).Size(new Vector2(-470, 50)).Build();

            new ButtonBuilder(this, "Copy Fields", CopyFields)
            .Parent(backDrop.transform).Location(new Vector2(445, -1370)).Size(new Vector2(-470, 50)).Build();

            assignToComponentBackground = DrawBackdrop("AssignToBackground", backDrop.transform, new Vector2(15, 85), new Vector2(-15, -1390), new Color(1, 1, 1, 0.128f));

            new ButtonBuilder(this, "Assign To", AssignTo)
            .Parent(backDrop.transform).Location(new Vector2(25, -1450)).Size(new Vector2(-470, 50)).Build();

            new DropDownBuilder(this).Parent(backDrop.transform).Location(new Vector2(235, -1450)).Size(new Vector2(-470, 50))
            .Build(assignTo, 200);

            if (assignTo.val == AssignToType.Group)
            {
                assignToName.choices = script.groupDictionary.Keys.ToList();
                prepareAssignNameChoices();
            }
            else if (assignTo.val == AssignToType.Layer)
            {
                assignToName.choices = script.actionsDictionary.Values.Select(item => item.Layer).Distinct().ToList();
                prepareAssignNameChoices();
            }
            if (assignToName.choices.Count > 0)
            {
                assignToName.val = assignToName.choices[0];
            }

            new DropDownBuilder(this).Parent(backDrop.transform).Location(new Vector2(445, -1450)).Size(new Vector2(-470, 50))
            .Build(assignToName, 200);

            new ButtonBuilder(this, "Action Routing", RoutingToggle)
            .Parent(backDrop.transform).Color(Color.white).Location(new Vector2(25, -1525)).Size(new Vector2(-45, 50)).Build();

        }
        else
        {
            string messageText = "";
            if (markType == MarkType.REMOVE)
            {
                messageText = "Remove " + markedActionNames.Count() + " marked actions";
            }
            else if (markType == MarkType.COPY_ROUTE)
            {
                messageText = "Copy routing of action '" + GetChosenAction() + "' to " + markedActionNames.Count() + " marked actions";
            }
            else if (markType == MarkType.COPY_FIELDS)
            {
                messageText = "Copy main fields from action '" + GetChosenAction() + "' to " + markedActionNames.Count() + " marked actions";
            }
            else if (markType == MarkType.ASSIGN_TO_GROUP)
            {
                messageText = "Assign " + markedActionNames.Count() + " marked actions to '" + assignToName.val + "' group";
            }
            else if (markType == MarkType.ASSIGN_TO_LAYER)
            {
                messageText = "Assign " + markedActionNames.Count() + " marked actions to '" + assignToName.val + "' layer";
            }
            else if (markType == MarkType.ROUTING)
            {
                messageText = "Selected actions will be linked to '" + GetChosenAction() + "' after applying";
            }


            new TitleBuilder(this, messageText)
            .FontSize(25).LabelColor(Color.white).Alignment(TextAnchor.MiddleCenter).Parent(backDrop.transform).Location(new Vector2(20, -1420)).Size(new Vector2(-30, 100)).Build();

            new ButtonBuilder(this, "Apply", ApplyMarked)
            .Parent(backDrop.transform).Color(new Color(0.45f, 1f, 0.45f)).Location(new Vector2(60, -1500)).Size(new Vector2(-470, 50)).Build();

            new ButtonBuilder(this, "Cancel", CancelMarked)
            .Parent(backDrop.transform).Color(Colors.REMOVE_COLOR).Location(new Vector2(400, -1500)).Size(new Vector2(-470, 50)).Build();
        }
    }

    private void RoutingToggle()
    {
        markType = MarkType.ROUTING;

        markedActionNames.AddRange(script.GetChoosenItem().LinkedActions);

        RefreshUI();
    }

    private void prepareAssignNameChoices()
    {
        assignToName.choices.Sort();
        assignToName.choices = assignToName.choices.Select(item => item == "" ? "<empty>" : item).ToList();
        if (!assignToName.choices.Contains("<empty>")) assignToName.choices.Insert(0, "<empty>");
    }

    private void ApplyMarked()
    {
        if (markType == MarkType.REMOVE)
        {
            foreach (var actionName in markedActionNames)
            {
                script.RemoveAction(actionName);
            }
        }
        else if (markType == MarkType.COPY_ROUTE)
        {
            ActionItem copyFrom = script.GetChoosenItem();
            foreach (var actionName in markedActionNames)
            {
                ActionItem targetActionItem = script.GetItemByName(actionName);
                targetActionItem.LinkedActions = copyFrom.LinkedActions.Where(item => item != actionName).ToList();
                if (targetActionItem.LinkedActions.Count() > 0)
                {
                    targetActionItem.Layer = "";
                }
            }
        }
        else if (markType == MarkType.COPY_FIELDS)
        {
            ActionItem copyFrom = script.GetChoosenItem();
            foreach (var actionName in markedActionNames)
            {
                script.GetItemByName(actionName).CopyFieldValues(copyFrom);
            }
        }
        else if (markType == MarkType.ASSIGN_TO_GROUP)
        {
            foreach (var actionName in markedActionNames)
            {
                if (assignToName.val == "<empty>")
                {
                    script.GetItemByName(actionName).Group = "";
                }
                else
                {
                    script.GetItemByName(actionName).Group = assignToName.val;
                }
            }
        }
        else if (markType == MarkType.ASSIGN_TO_LAYER)
        {
            foreach (var actionName in markedActionNames)
            {
                if (assignToName.val == "<empty>")
                {
                    script.GetItemByName(actionName).Layer = "";
                }
                else
                {
                    script.GetItemByName(actionName).Layer = assignToName.val;
                }

                script.GetItemByName(actionName).LinkedActions.Clear();
            }
        }
        else if (markType == MarkType.ROUTING)
        {
            script.GetChoosenItem().LinkedActions = new List<string>(markedActionNames);
            UI.linkedActionsUI.RefreshUI();
        }

        UI.newLinkedActionsUI.RefreshUI();
        UI.actionTextFieldsUI.RefreshUI();

        CancelMarked();
    }

    private void CancelMarked()
    {
        markedActionNames.Clear();
        markType = MarkType.NO_MARK;
        RefreshUI();
    }

    private bool isMarkMode()
    {
        return markType != MarkType.NO_MARK;
    }

    private string GetGroupExpansionKey(string groupName)
    {
        return groupBy + "-" + groupName;
    }
}
