using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Experimental.PlayerLoop;
using UnityEngine.UI;

public class InspectorUI : UIPart
{
    private bool isOpen = false;
    private GameObject inspectorBackdropShadow;
    private GameObject inspectorBackdrop;

    private Scroll chatScroll;

    private Scroll actionScroll;

    public List<Message> messages = new List<Message>();
    private UIDynamicButton voxtaStateButton;

    private JSONStorableString chatText;

    private UIDynamicLabelInput chatInput;
    private UIDynamicButton rollbackButton;
    private UIDynamicButton clearButton;
    private UIDynamicButton sendButton;
    private List<UIDynamic> uiMessages = new List<UIDynamic>();
    private UIDynamicTextInfo contextField;

    public InspectorUI(ActionListManager script) : base(script) { }

    public override void RefreshUI()
    {
        if (!isOpen)
        {
            return;
        }

        RemoveAllUI();
        voxtaStateButton = new ButtonBuilder(this, Voxta.IsActive(script) ? "Voxta active" : "Voxta inactive", OnVoxtaToggle).Color(Voxta.IsActive(script) ? Colors.BUTTON_TRANSITION_ACTION : Colors.BUTTON_DANGER).Parent(actionScroll.scrollContent).Build();
        UpdateVoxtaState();

        var spacer = script.CreateSpacer();
        spacer.transform.SetParent(actionScroll.scrollContent);
        spacer.height = 5;
        uiElements.Add(spacer);

        new TitleBuilder(this, "Last Action").Parent(actionScroll.scrollContent).Alignment(TextAnchor.MiddleCenter).LabelColor(Color.black).FontBold().FontSize(30).Build();
        new ButtonBuilder(this, script.lastAction != null ? script.lastAction.Name : "No Action Executed", null).Parent(actionScroll.scrollContent).Build();

        spacer = script.CreateSpacer();
        spacer.transform.SetParent(actionScroll.scrollContent);
        spacer.height = 10;
        uiElements.Add(spacer);
        new TitleBuilder(this, "Available Actions").Parent(actionScroll.scrollContent).Alignment(TextAnchor.MiddleCenter).LabelColor(Color.black).FontBold().FontSize(30).Build();

        if (script.lastRoutingAction != null &&
        (script.IsActionExists(script.lastRoutingAction.Name) || script.lastRoutingAction.Name == ActionItem.BOOTSTRAP_NAME))
        {
            List<string> routedActions = new List<string>(); routedActions.AddRange(script.lastRoutingAction.LinkedActions); routedActions.AddRange(script.GetGlobalActionNames());

            foreach (var actionName in routedActions)
            {
                ActionItem actionItem = script.GetItemByName(actionName);
                if (actionItem.UserAction)
                {
                    continue;
                }
                if (script.lockedOnTransitionAction && !actionItem.IsLayeredAction())
                {
                    continue;
                }

                Color actionButtonColor = Colors.BUTTON_DEFAULT;
                if (actionItem.TransitionAction)
                {
                    actionButtonColor = Colors.BUTTON_TRANSITION_ACTION;
                }
                else if (actionItem.IsLayeredAction())
                {
                    actionButtonColor = Colors.BUTTON_LAYERED_ACTION;
                }
                new ButtonBuilder(this, actionName, () => script.ExecuteAction(actionItem)).Color(actionButtonColor).Parent(actionScroll.scrollContent).Build();
            }
        }
    }

    public void UpdateVoxtaState()
    {
        if (isOpen)
        {
            string voxtaState = Voxta.GetState(script);
            if (voxtaState == "off")
            {
                voxtaStateButton.buttonColor = Colors.BUTTON_DANGER;
                voxtaStateButton.buttonText.text = "Voxta inactive";
            }
            else if (voxtaState == "idle")
            {
                voxtaStateButton.buttonColor = Colors.BUTTON_TRANSITION_ACTION;
                voxtaStateButton.buttonText.text = "Voxta active";
            }
            else
            {
                voxtaStateButton.buttonColor = Colors.BUTTON_LAYERED_ACTION;
                voxtaStateButton.buttonText.text = "Voxta " + voxtaState;
            }
        }
    }

    public void Open()
    {
        if (!isOpen)
        {
            InitUI();
        }
    }

    public override void Init()
    {
        chatText = new JSONStorableString("ChatText", "", OnUserMessageTypedIn);
    }

    private void InitUI()
    {
        inspectorBackdropShadow = new GameObject("Backdrop Shadow");
        inspectorBackdropShadow.transform.SetParent(script.UITransform, false);

        var rect = inspectorBackdropShadow.AddComponent<RectTransform>();
        rect.anchorMin = Vector2.zero;
        rect.anchorMax = Vector2.one;
        rect.offsetMin = new Vector2(430, -30f);
        rect.offsetMax = new Vector2(830f, -330f);

        var bg = inspectorBackdropShadow.AddComponent<Image>();
        bg.color = new Color(0, 0, 0, 0.5f);

        inspectorBackdrop = new GameObject("Backdrop Inspector");
        inspectorBackdrop.transform.SetParent(script.UITransform, false);

        var rect2 = inspectorBackdrop.AddComponent<RectTransform>();
        rect2.anchorMin = Vector2.zero;
        rect2.anchorMax = Vector2.one;
        rect2.offsetMin = new Vector2(400, 0f);
        rect2.offsetMax = new Vector2(800f, -300f);

        bg = inspectorBackdrop.AddComponent<Image>();
        bg.color = new Color(1, 1, 1, 0.8f);

        new ButtonBuilder(this, "X", () => { UnityEngine.Object.Destroy(inspectorBackdropShadow); UnityEngine.Object.Destroy(inspectorBackdrop); chatScroll.Destroy(); isOpen = false; })
        .Color(Colors.BUTTON_DANGER).Location(new Vector2(1400, -55)).Size(new Vector2(-1405, 50)).Parent(inspectorBackdrop.transform).ConstantElement().Build();

        UIDynamicButton moveButton = new ButtonBuilder(this, "Inspector", null)
        .Color(Colors.BUTTON_DEFAULT).Location(new Vector2(8, -55)).Size(new Vector2(-95, 50)).Parent(inspectorBackdrop.transform).ConstantElement().Build();
        var colors = moveButton.button.colors;
        colors.highlightedColor = Colors.BUTTON_DEFAULT;
        colors.pressedColor = Colors.BUTTON_DEFAULT;
        moveButton.button.colors = colors;

        chatScroll = new Scroll(inspectorBackdrop.transform, "ChatScroll", 0.26f, 0.99f, 0.78f);
        actionScroll = new Scroll(inspectorBackdrop.transform, "ActionScroll", 0.01f, 0.25f);

        chatInput = new TextInputBuilder(script, uiElements, "Chat", chatText).LabelColor(Color.black).Location(new Vector2(0, -960)).Size(new Vector2(1210, 50)).Parent(rect2).ConstantElement().Build();
        rollbackButton = new ButtonBuilder(this, "ROL", OnRollbackClicked)
        .Color(Colors.BUTTON_WARNING).Location(new Vector2(1305, -965)).Size(new Vector2(-1405, 50)).Parent(inspectorBackdrop.transform).ConstantElement().Build();
        clearButton = new ButtonBuilder(this, "CLR", OnClearClicked)
        .Color(Colors.BUTTON_DANGER).Location(new Vector2(1390, -965)).Size(new Vector2(-1405, 50)).Parent(inspectorBackdrop.transform).ConstantElement().Build();
        sendButton = new ButtonBuilder(this, "SEND", () => SendChatMessage(chatText.val)).Location(new Vector2(1204, -965)).Size(new Vector2(-1390, 50))
        .Parent(inspectorBackdrop.transform).ConstantElement().Build();

        UpdateInputs(Voxta.GetState(script) != "off");

        contextField = Utils.SetupInfoTextNoScroll(script, Context.GetContextForDisplay(Voxta.GetCurrentContext(script)), 40, false);
        contextField.transform.SetParent(inspectorBackdrop.transform);
        contextField.transform.Find("Background").GetComponent<Image>().color = new Color(0.6392156863f, 0.6392156863f, 0.6392156863f, 0.3f);
        contextField.transform.Find("Text").GetComponent<Text>().alignment = TextAnchor.MiddleLeft;
        contextField.transform.Find("Text").GetComponent<Text>().fontSize = 26;
        var rectTransform = contextField.GetComponent<RectTransform>();
        rectTransform.pivot = Vector2.zero;
        rectTransform.anchoredPosition = new Vector2(385, 790);
        rectTransform.sizeDelta = new Vector2(-400, -870);

        DrawMessages();
        chatScroll.ScrollDown();
        isOpen = true;
        RefreshUI();

        var handler = moveButton.gameObject.AddComponent<UndockHandler>();
        handler.Initialize(rect);

        inspectorBackdrop.transform.SetParent(inspectorBackdropShadow.transform);
    }

    private void DrawMessages()
    {
        foreach (var message in messages)
        {
            if (message.type == Message.USER_TYPE)
            {
                RenderMessage(message.text, Colors.USER_MESSAGE, TextAnchor.MiddleRight);
            }
            else
            {
                RenderMessage(message.text, Colors.CHARACTER_MESSAGE, TextAnchor.MiddleLeft);
            }
        }
    }

    private void OnRollbackClicked()
    {
        Voxta.RevertLastSentMessage(script);
        RollbackDisplayedMessages();
        RefreshChatMessages();
    }

    private void OnClearClicked()
    {
        messages.Clear();
        RefreshChatMessages();
        Voxta.DeleteCurrentChat(script);
    }

    private void OnVoxtaToggle()
    {
        string voxtaState = Voxta.GetState(script);
        Voxta.ToggleOn(script);
        UpdateInputs(voxtaState == "off");
        RefreshUI();
    }

    private void UpdateInputs(bool enabled)
    {
        chatInput.input.enabled = enabled;
        rollbackButton.button.enabled = enabled;
        clearButton.button.enabled = enabled;
    }

    private void OnUserMessageTypedIn(string text)
    {
        if (Input.GetKeyDown(KeyCode.Return) || Input.GetKeyDown(KeyCode.KeypadEnter))
        {
            SendChatMessage(text);
        }
    }

    private void SendChatMessage(string text)
    {
        if (!string.IsNullOrEmpty(text))
        {
            chatText.valNoCallback = "";
            chatInput.input.ActivateInputField();
            ActionListManager.getPluginByType(script.containingAtom, "Voxta").GetStringJSONParam("TriggerMessage").val = text;
            OnIncomingUserMessage(text);
        }
    }

    public void RenderMessage(string text, Color backgroundColor, TextAnchor alignment)
    {
        var info = Utils.SetupInfoTextNoScroll(script, text, 40, false);
        info.transform.Find("Background").GetComponent<Image>().color = backgroundColor;
        info.transform.Find("Text").GetComponent<Text>().alignment = alignment;
        info.transform.SetParent(chatScroll.scrollContent);
        uiMessages.Add(info);
    }

    public void OnIncomingUserMessage(string text)
    {
        if (isOpen)
        {
            RenderMessage(text, Colors.USER_MESSAGE, TextAnchor.MiddleRight);
            chatScroll.ScrollDown();
        }
        messages.Add(new Message(Message.USER_TYPE, text));
    }

    public void OnIncomingCharacterMessage(string text)
    {
        if (isOpen)
        {
            RenderMessage(text, Colors.CHARACTER_MESSAGE, TextAnchor.MiddleLeft);
            chatScroll.ScrollDown();
        }
        messages.Add(new Message(Message.CHARACTER_TYPE, text));
    }

    public void UpdateCurrentContext(string context)
    {
        if (isOpen)
        {
            contextField.text.text = Context.GetContextForDisplay(context);
        }
    }

    private void RollbackDisplayedMessages()
    {
        bool userFound = false;
        int indexToRemove = 0;

        for (int i = messages.Count - 1; i >= 0; i--)
        {
            if (messages[i].type == Message.USER_TYPE)
            {
                userFound = true;
                indexToRemove = i;
                break;
            }
        }

        if (userFound)
        {
            messages.RemoveRange(indexToRemove, messages.Count - indexToRemove);
        }
    }

    private void RefreshChatMessages()
    {
        Utils.RemoveUIElements(script, uiMessages.Cast<object>().ToList());
        uiMessages.Clear();
        DrawMessages();
    }

    public void Destroy()
    {
        UnityEngine.Object.Destroy(inspectorBackdropShadow); UnityEngine.Object.Destroy(inspectorBackdrop);
    }
}
