using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using UnityEngine.Events;

using System.Collections;
using System;
using static MainUI;
using UnityEngine.UI;

public class LinkedActionsUI : UIPart
{
    public bool showAdvancedSettings = false;
    public JSONStorableBool sameActionNameFunction;
    public JSONStorableString layerField;
    public JSONStorableBool transitionActionField;
    public JSONStorableBool userActionField;
    public JSONStorableBool playTimelineAnimField;
    private JSONStorableBool deferredField;
    private Dictionary<string, ActionItem> actionsDictionary;

    public LinkedActionsUI(ActionListManager script) : base(script)
    {
        actionsDictionary = script.actionsDictionary;
    }

    public override void Init()
    {
        sameActionNameFunction = new JSONStorableBool("Same Function And Name", true, OnSameFunctionAndNameChanged);
        layerField = new JSONStorableString("Layer", "", OnLayerChanged);
        transitionActionField = new JSONStorableBool("Transition Action", false, OnTransitionActionChanged);
        userActionField = new JSONStorableBool("User Action", false, OnUserActionChanged);
        playTimelineAnimField = new JSONStorableBool("Play Timeline Animation", false, OnPlayTimelineAnimationChanged);
        deferredField = new JSONStorableBool("When Speech Finished", false, OnDeferredFieldChanged);
    }
    override
    public void RefreshUI()
    {
        RemoveAllUI();

        if (!script.lockedOnTransitionAction)
        {
            string currentSelectedAction = GetChosenAction();
            if (string.IsNullOrEmpty(currentSelectedAction))
            {
                return;
            }

            ActionItem currentlySelectedActionItem = actionsDictionary[currentSelectedAction];

            if (!currentlySelectedActionItem.UserAction)
            {
                if (currentlySelectedActionItem.IsEmptyRouting())
                {
                    uiElements.Add(Utils.CreateTitle(script, "Layered actions can't have any links", true, 24, true, true));
                    uiElements.Add(Utils.CreateTitle(script, "and are executed alongside normal ones", true, 24, true, true));

                    JSONStorableStringChooser layerChooser = new JSONStorableStringChooser("Layer", new List<string> { "" }, script.GetChoosenItem().Layer, "Layer", OnLayerChanged);
                    layerChooser.choices.AddRange(script.layerDictionary.Keys.ToList());
                    new DropDownWithCreateNewBuilder(this).Parent(script.GetRightUIContent()).Build(layerChooser, () => { UI.layerUI.Open(script.GetChoosenItem().Layer); });
                }
                uiElements.Add(Utils.CreateTitle(script, "Execute action after done talking", true, 24, true, true));
                uiElements.Add(Utils.SetupToggle(script, deferredField, true));
            }

            uiElements.Add(Utils.CreateTitle(script, "User Actions can be called from UIAssist", true, 24, true, true));
            uiElements.Add(Utils.SetupToggle(script, userActionField, true));

            uiElements.Add(Utils.SetupSpacer(script, 5, true));

            if (!currentlySelectedActionItem.UserAction)
            {
                uiElements.Add(Utils.SetupButton(script, showAdvancedSettings ? "↑    Advanced Settings    ↑" : "↓    Advanced Settings    ↓", OnShowAdvancedClicked, true));
                if (showAdvancedSettings)
                {
                    {
                        uiElements.Add(Utils.CreateTitle(script, "Allows to define separate function name", true, 24, true, true));
                        uiElements.Add(Utils.CreateTitle(script, "that is sent to LLM", true, 24, true, true));
                        uiElements.Add(Utils.SetupToggle(script, sameActionNameFunction, true));
                        uiElements.Add(Utils.CreateTitle(script, "Locks action execution with the exclusion", true, 24, true, true));
                        uiElements.Add(Utils.CreateTitle(script, "of layered actions", true, 24, true, true));
                        uiElements.Add(Utils.SetupToggle(script, transitionActionField, true));
                    }

                    uiElements.Add(Utils.CreateTitle(script, "On execution plays a Timeline animation", true, 24, true, true));
                    uiElements.Add(Utils.CreateTitle(script, "with name 'VA_' + Function Name", true, 24, true, true));
                    uiElements.Add(Utils.SetupToggle(script, playTimelineAnimField, true));
                    uiElements.Add(Utils.CreateTitle(script, "Link all actions in this group together", true, 24, true, true));
                    uiElements.Add(Utils.SetupButton(script, "Link Group", OnLinkAllActionsInGroup, true));
                }
            }

            if (!string.IsNullOrEmpty(GetChosenAction()))
            {
                copyFieldValues(script.GetChoosenItem());
            }
            else
            {
                resetAllFields();
            }
        }
    }

    private void OnLinkAllActionsInGroup()
    {
        ActionItem currentActionItem = script.GetItemByName(GetChosenAction());
        string group = currentActionItem.Group;

        List<ActionItem> groupedActionItems = new List<ActionItem>();
        foreach (var actionItem in actionsDictionary.Values.ToArray())
        {
            if (actionItem.Group == group)
            {
                groupedActionItems.Add(actionItem);
            }
        }

        foreach (var actionItem in groupedActionItems)
        {
            foreach (var itemToAdd in groupedActionItems)
            {
                if (itemToAdd != actionItem && !actionItem.LinkedActions.Contains(itemToAdd.Name))
                {
                    LinkActionItemToName(actionItem, itemToAdd.Name);
                }
            }
        }

        showAdvancedSettings = false;
        RefreshUI();
    }

    public void ApplySameFunctionNameToggle(ActionItem actionItem)
    {
        sameActionNameFunction.val = actionItem.Name == actionItem.Action;
    }
    private void OnTransitionActionChanged(bool value)
    {
        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].TransitionAction = value;
        }

        UI.bulkActionManagerUI.RefreshUI();
    }
    private void OnUserActionChanged(bool value)
    {
        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].UserAction = value;
            script.RefreshActionUI();
        }
    }
    private void OnPlayTimelineAnimationChanged(bool value)
    {
        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].PlayTimelineAnim = value;
        }
    }
    private void OnSameFunctionAndNameChanged(bool value)
    {
        if (!value)
        {
            UI.actionTextFieldsUI.OnNameFieldChanged(ActionNameUtils.HumanFormatActionName(GetChosenAction()));
        }
        else
        {
            script.GetChoosenItem().Action = ActionNameUtils.FunctionFormatActionName(GetChosenAction());
            sameActionNameFunction.val = true;
        }

        UI.actionTextFieldsUI.RefreshUI();
    }

    private void OnLayerChanged(string newValue)
    {
        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            script.GetChoosenItem().Layer = newValue;
        }

        UI.bulkActionManagerUI.RefreshUI();
    }

    private void OnDeferredFieldChanged(bool newValue)
    {
        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Deferred = newValue;
        }
    }

    public void LinkActionItemToName(ActionItem actionItem, string nameToAdd)
    {
        if (!ActionItemHasSameActionName(actionItem, nameToAdd))
        {
            if (!string.IsNullOrEmpty(actionItem.Layer))
            {
                actionItem.Layer = "";
            }
            actionItem.LinkedActions.Add(nameToAdd);
        }
        else
        {
            SuperController.LogError("Action with function name = " + actionsDictionary[nameToAdd].Action + " already linked. Cannot have duplicate function names in the same route");
        }
    }

    private bool ActionItemHasSameActionName(ActionItem actionItem, string nameToAdd)
    {
        foreach (string name in actionItem.LinkedActions)
        {
            if (actionsDictionary[name].Action == actionsDictionary[nameToAdd].Action)
            {
                return true;
            }
        }
        return false;
    }

    public void resetAllFields()
    {
        layerField.val = "";
        transitionActionField.val = false;
        userActionField.valNoCallback = false;
        playTimelineAnimField.val = true;
        deferredField.val = false;
    }

    public void copyFieldValues(ActionItem actionItem)
    {
        if (actionItem != null && layerField != null)
        {
            layerField.val = actionItem.Layer;
            transitionActionField.val = actionItem.TransitionAction;
            userActionField.valNoCallback = actionItem.UserAction;
            playTimelineAnimField.val = actionItem.PlayTimelineAnim;
            deferredField.val = actionItem.Deferred;
        }
    }

    private void OnShowAdvancedClicked()
    {
        showAdvancedSettings = !showAdvancedSettings;
        RefreshUI();
    }
}
