using UnityEngine;
using UnityEngine.UI;

public class Scroll
{
    public RectTransform scrollContent { get; private set; }

    public GameObject scrollView { get; private set; }

    public Scroll(Transform parent, string id, float leftWidth, float rightWidth, float topHeight = 0.92f)
    {
        scrollView = new GameObject(id);
        var scrollViewRectTransform = scrollView.AddComponent<RectTransform>();
        scrollViewRectTransform.SetParent(parent, false);

        var scrollRect = scrollView.AddComponent<ScrollRect>();

        scrollViewRectTransform.anchorMin = new Vector2(leftWidth, 0.12f);
        scrollViewRectTransform.anchorMax = new Vector2(rightWidth, topHeight);

        scrollViewRectTransform.offsetMin = Vector2.zero;
        scrollViewRectTransform.offsetMax = Vector2.zero;
        scrollView.AddComponent<Image>();
        scrollView.AddComponent<Mask>().showMaskGraphic = false;

        scrollContent = new GameObject("Content").AddComponent<RectTransform>();
        scrollContent.SetParent(scrollViewRectTransform.transform, false);
        scrollContent.pivot = new Vector2(0.5f, 1f);
        scrollContent.anchorMin = Vector2.zero;
        scrollContent.anchorMax = Vector2.one;
        scrollContent.offsetMin = Vector2.zero;
        scrollContent.offsetMax = Vector2.zero;
        var contentSizeFitter = scrollContent.gameObject.AddComponent<ContentSizeFitter>();
        contentSizeFitter.verticalFit = ContentSizeFitter.FitMode.PreferredSize;

        scrollRect.content = scrollContent;
        scrollRect.vertical = true;
        scrollRect.horizontal = false;

        var layoutGroup = scrollContent.gameObject.AddComponent<VerticalLayoutGroup>();
        layoutGroup.childControlHeight = true;
        // layoutGroup.childForceExpandHeight = false;
        // layoutGroup.childAlignment = TextAnchor.UpperLeft;
        layoutGroup.spacing = 10;

    }

    public void ScrollDown()
    {
        scrollView.GetComponent<ScrollRect>().verticalNormalizedPosition = -1f;
    }

    public void Destroy()
    {
        UnityEngine.Object.Destroy(scrollView);
        UnityEngine.Object.Destroy(scrollContent.gameObject);
    }
}
