using UnityEngine;
using UnityEngine.UI;

public static class UIHelpers
{
    public static Transform InstantiateButton(this MVRScript script, Transform parent = null) =>
            Instantiate(script.manager.configurableButtonPrefab, parent);

    public static Transform InstantiatePopup(this MVRScript script, Transform parent = null) =>
    Instantiate(script.manager.configurableFilterablePopupPrefab, parent);

    public static Transform InstantiateTextField(this MVRScript script, Transform parent = null) =>
    Instantiate(script.manager.configurableTextFieldPrefab, parent);

    public static Transform InstantiateInputField(this MVRScript script, Transform parent = null) =>
    Instantiate(script.manager.configurableScrollablePopupPrefab, parent);

    static Transform Instantiate(Transform prefab, Transform parent = null)
    {
        var transform = UnityEngine.Object.Instantiate(prefab, parent, false);
        UnityEngine.Object.Destroy(transform.GetComponent<LayoutElement>());
        return transform;
    }

    static RectTransform GetRectTransform(Transform transform)
    {
        var rectTransform = transform.GetComponent<RectTransform>();
        rectTransform.pivot = Vector2.zero;
        return rectTransform;
    }

    public static UIDynamicButton CreateCustomButton(MVRScript script, string label, Vector2 position, Vector2 size)
    {
        var t = script.InstantiateButton(script.UITransform);
        var rectT = GetRectTransform(t);
        rectT.anchoredPosition = position;
        rectT.sizeDelta = size;
        var uiDynamic = t.GetComponent<UIDynamicButton>();
        uiDynamic.label = label;

        // uiDynamic.textColor = Color.white;
        // uiDynamic.buttonColor = Color.gray;
        // var colors = uiDynamic.button.colors;
        // colors.highlightedColor = Color.white;
        // colors.pressedColor = Color.white;
        // uiDynamic.button.colors = colors;

        return uiDynamic;
    }

    public static UIDynamicPopup CreateCustomPopup(MVRScript script, string label, Vector2 position, Vector2 size)
    {
        var t = script.InstantiatePopup(script.UITransform);
        var rectT = GetRectTransform(t);
        rectT.anchoredPosition = position;
        rectT.sizeDelta = size;
        var uiDynamic = t.GetComponent<UIDynamicPopup>();
        uiDynamic.label = label;
        return uiDynamic;
    }

    public static void DrawUILine(RectTransform parent, Vector2 startPos, Vector2 endPos, float width, Color color)
    {
        GameObject line = new GameObject("UILine");
        line.transform.SetParent(parent, false);

        Image lineImage = line.AddComponent<Image>();
        lineImage.color = color;

        RectTransform rectTransform = line.GetComponent<RectTransform>();
        Vector2 direction = (endPos - startPos).normalized;
        float distance = Vector2.Distance(startPos, endPos);

        rectTransform.sizeDelta = new Vector2(distance, width);
        rectTransform.anchorMin = rectTransform.anchorMax = Vector2.zero;
        rectTransform.pivot = new Vector2(0, 0.5f);
        rectTransform.localPosition = startPos;
        rectTransform.localRotation = Quaternion.Euler(0, 0, Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg);
    }
}
