using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

public abstract class UIPart
{
    public List<UIDynamic> uiElements { get; private set; } = new List<UIDynamic>();

    public ActionListManager script { get; private set; }

    protected UIPart(ActionListManager script)
    {
        this.script = script;
        Init();
    }

    public abstract void Init();

    public abstract void RefreshUI();

    public void RemoveAllUI()
    {
        Utils.RemoveUIElements(script, uiElements.Cast<object>().ToList());
        uiElements.Clear();
    }

    public GameObject DrawBackdrop(string name, Transform parent, Vector2 backDropOffsetMin, Vector2 backDropOffsetMax, Color color, int frameWidth = 0)
    {
        GameObject frameObject = new GameObject("Frame");
        frameObject.transform.SetParent(parent, false);

        var frameRect = frameObject.AddComponent<RectTransform>();
        frameRect.anchorMin = Vector2.zero;
        frameRect.anchorMax = Vector2.one;
        frameRect.offsetMin = backDropOffsetMin; // Tutaj definiujesz "grubość" ramki poprzez zewnętrzny margines
        frameRect.offsetMax = backDropOffsetMax;

        var frameImage = frameObject.AddComponent<Image>();
        frameImage.color = frameWidth != 0 ? Color.white : new Color(1, 1, 1, 0); // Kolor ramki

        // Tworzenie obiektu backDrop jako "wewnętrzne tło" wewnątrz ramki
        GameObject backDrop = new GameObject(name);
        backDrop.transform.SetParent(frameObject.transform, false); // Teraz rodzicem jest frameObject

        var rect = backDrop.AddComponent<RectTransform>();
        rect.anchorMin = Vector2.zero;
        rect.anchorMax = Vector2.one;
        rect.offsetMin = new Vector2(frameWidth, frameWidth); // Odstęp od ramki, tworzy wewnętrzną zawartość mniejszą niż ramka
        rect.offsetMax = new Vector2(-frameWidth, -frameWidth);

        var bg = backDrop.AddComponent<Image>();
        bg.color = color; // Kolor wewnętrznej części (zawartości)

        return frameObject;
    }

    public RectTransform CreateScroll(Transform parent, Vector2 anchorMin, Vector2 anchorMax, int spacing = 10)
    {
        var scrollView = new GameObject("ScrollView").AddComponent<RectTransform>();
        scrollView.SetParent(parent, false);

        var scrollRect = scrollView.gameObject.AddComponent<ScrollRect>();
        scrollView.anchorMin = anchorMin;
        scrollView.anchorMax = anchorMax;
        scrollView.offsetMin = Vector2.zero;
        scrollView.offsetMax = Vector2.zero;
        scrollView.gameObject.AddComponent<Mask>().showMaskGraphic = false;
        scrollView.gameObject.AddComponent<Image>();

        var scrollContent = new GameObject("Content").AddComponent<RectTransform>();
        scrollContent.SetParent(scrollView.transform, false);
        scrollContent.pivot = new Vector2(0.5f, 1f);
        scrollContent.anchorMin = Vector2.zero;
        scrollContent.anchorMax = Vector2.one;
        scrollContent.offsetMin = Vector2.zero;
        scrollContent.offsetMax = Vector2.zero;
        var contentSizeFitter = scrollContent.gameObject.AddComponent<ContentSizeFitter>();
        contentSizeFitter.verticalFit = ContentSizeFitter.FitMode.PreferredSize;

        scrollRect.content = scrollContent;
        scrollRect.vertical = true;
        scrollRect.horizontal = false;

        var layoutGroup = scrollContent.gameObject.AddComponent<VerticalLayoutGroup>();
        layoutGroup.childControlHeight = true;
        layoutGroup.childForceExpandHeight = false;
        layoutGroup.childAlignment = TextAnchor.UpperLeft;
        layoutGroup.spacing = spacing;

        return scrollContent;
    }
}
