using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class ButtonBuilder : UIBuilder<ButtonBuilder>
{
    private string label;
    private int fontSize = 30;
    private bool fontBold = false;
    private bool otherStyle = false;
    private UnityAction callback;
    private Vector2 location;
    private Vector2 size;
    private Color color = Colors.BUTTON_DEFAULT;

    public ButtonBuilder(UIPart uIPart, string label, UnityAction callback) : base(uIPart)
    {
        this.label = label;
        this.callback = callback;
    }
    public ButtonBuilder FontSize(int fontSize)
    {
        this.fontSize = fontSize;
        return this;
    }
    public ButtonBuilder FontBold()
    {
        fontBold = true;
        return this;
    }
    public ButtonBuilder FontBold(bool isBold)
    {
        fontBold = isBold;
        return this;
    }
    public ButtonBuilder Color(Color color)
    {
        this.color = color;
        return this;
    }
    public ButtonBuilder Location(Vector2 location)
    {
        this.location = location;
        return this;
    }
    public ButtonBuilder Size(Vector2 size)
    {
        this.size = size;
        return this;
    }
    public UIDynamicButton Build()
    {
        var button = Utils.SetupButton(script, label, callback, rightSide);
        button.buttonColor = color;

        if (fontBold)
        {
            button.buttonText.fontStyle = FontStyle.Bold;
        }

        FinalizeBuild(button);
        var rectTransform = button.GetComponent<RectTransform>();
        LayoutElement le = button.gameObject.AddComponent<LayoutElement>();

        if (location != null)
        {
            rectTransform.pivot = Vector2.zero;
            rectTransform.anchoredPosition = location;
        }
        if (size != null)
        {
            rectTransform.sizeDelta = size;
        }

        return button;
    }
}
