using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class CustomWidthButtonBuilder : PrefabBasedBuilder<CustomWidthButtonBuilder>
{
    private string label;
    private int fontSize = 30;
    private bool fontBold = true;
    private bool otherStyle = false;
    private UnityAction callback;
    private Color buttonColor = Colors.BUTTON_DEFAULT;
    private Color textColor = UnityEngine.Color.black;
    private string additionalLabel;

    public CustomWidthButtonBuilder(UIPart uIPart, string label, UnityAction callback) : base(uIPart)
    {
        this.label = label;
        this.callback = callback;
    }
    public CustomWidthButtonBuilder FontSize(int fontSize)
    {
        this.fontSize = fontSize;
        return this;
    }
    public CustomWidthButtonBuilder FontBold()
    {
        fontBold = true;
        return this;
    }
    public CustomWidthButtonBuilder FontBold(bool isBold)
    {
        fontBold = isBold;
        return this;
    }
    public CustomWidthButtonBuilder ButtonColor(Color buttonColor)
    {
        this.buttonColor = buttonColor;
        return this;
    }
    public CustomWidthButtonBuilder TextColor(Color textColor)
    {
        this.textColor = textColor;
        return this;
    }
    public CustomWidthButtonBuilder AdditionalLabel(string additionalLabel)
    {
        this.additionalLabel = additionalLabel;
        return this;
    }

    public override void CreatePrefab()
    {
        RectTransform buttonTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Button") as RectTransform;
        buttonTransform = Object.Instantiate(buttonTransform, customPrefab.transform);
        buttonTransform.name = "CustomWidthButton";
        buttonTransform.anchorMin = new Vector2(0.1f, 0.0f);
        buttonTransform.anchorMax = new Vector2(1.0f, 1.0f);
        buttonTransform.offsetMin = new Vector2(0, 0);
        buttonTransform.offsetMax = new Vector2(-3, 0);
        // Image buttonImage = buttonTransform.GetComponent<Image>();
        // buttonImage.color = buttonColor;

        RectTransform mainLabelTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Button/Text") as RectTransform;
        mainLabelTransform = Object.Instantiate(mainLabelTransform, customPrefab.transform);
        mainLabelTransform.name = "Text";
        mainLabelTransform.offsetMin = new Vector2(-480, 0);
        Text labelText = mainLabelTransform.GetComponent<Text>();
        labelText.color = Color.black;
        labelText.fontStyle = FontStyle.Bold;

        Button buttonLeft = buttonTransform.GetComponent<Button>();
        Text labelLeft = buttonTransform.Find("Text").GetComponent<Text>();

        UIDynamicCustomWidthButton uid = customPrefab.AddComponent<UIDynamicCustomWidthButton>();
        uid.labelLeft = labelLeft;
        uid.buttonLeft = buttonLeft;
        uid.labelAdditional = labelText;
    }

    public override UIDynamic BuildFromPrefab()
    {
        Transform t = Utils.ourCreateUIElement(customPrefab.transform, rightSide);
        UIDynamicCustomWidthButton uid = t.GetComponent<UIDynamicCustomWidthButton>();
        uid.labelLeft.text = label;
        uid.labelLeft.color = textColor;
        uid.labelAdditional.text = additionalLabel;

        RectTransform leftButtonRect = uid.buttonLeft.GetComponent<RectTransform>();
        leftButtonRect.anchorMin = new Vector2(0.1f, 0.0f);
        uid.buttonLeft.GetComponent<Image>().color = buttonColor;

        uid.buttonLeft.onClick.AddListener(callback);
        t.gameObject.SetActive(true);

        return uid;
    }

    public class UIDynamicCustomWidthButton : UIDynamicUtils
    {
        public Text labelLeft;
        public Button buttonLeft;
        public Text labelAdditional;
    }
}
