using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class DropDownBuilder : UIBuilder<DropDownBuilder>
{

    private Vector2 location;
    private Vector2 size;

    public DropDownBuilder(UIPart uIPart) : base(uIPart)
    {
    }

    public DropDownBuilder Location(Vector2 location)
    {
        this.location = location;
        return this;
    }
    public DropDownBuilder Size(Vector2 size)
    {
        this.size = size;
        return this;
    }

    public UIDynamicPopup Build(JSONStorableStringChooser jsc, float popupPanelHeight = 350f)
    {
        var prefab = script.manager.configurableScrollablePopupPrefab;


        var ui = UnityEngine.Object.Instantiate(prefab).GetComponent<UIDynamicPopup>();
        ui.gameObject.transform.SetParent(parent, false);
        ui.label = jsc.name;
        jsc.popup = ui.popup;

        script.RegisterStringChooser(jsc);

        UnityEngine.Object.Destroy(ui.transform.Find("Slider").gameObject);
        UnityEngine.Object.Destroy(ui.transform.Find("Background").gameObject);
        // UnityEngine.Object.Destroy(ui.transform.Find("Text").gameObject);

        // SuperController.LogError(ObjectHierarchyToString(prefab.transform));
        // ui.popup.labelText.alignment = TextAnchor.MiddleLeft;
        // ui.labelWidth = 120f;
        // ui.popup.labelText.fontSize = 24;
        // var labelTextRect = ui.popup.labelText.GetComponent<RectTransform>();
        // labelTextRect.offsetMin += new Vector2(10f, 0f);
        // ui.popup.labelText.GetComponent<RectTransform>().anchorMax = new Vector2(0.04f, 0.91f);

        var popupLayout = ui.popup.GetComponent<LayoutElement>();
        popupLayout.minHeight = 60f;
        popupLayout.preferredHeight = 60f;

        var topButtonRect = ui.popup.topButton.GetComponent<RectTransform>();
        // topButtonRect.anchoredPosition = new Vector2(0f, 0f);
        topButtonRect.offsetMin = new Vector2(0f, 0f);
        topButtonRect.offsetMax = new Vector2(0f, 0f);

        var dropDownPanel = ui.transform.Find("PopupPanel").GetComponent<RectTransform>();
        dropDownPanel.anchoredPosition = new Vector2(0f, 0f);
        dropDownPanel.offsetMin = new Vector2(0f, 0f);
        dropDownPanel.offsetMax = new Vector2(0f, 0f);

        ui.labelTextColor = Color.black;
        ui.label = "↓";
        var labelTransform = ui.transform.Find("Text").GetComponent<RectTransform>();
        labelTransform.anchoredPosition = new Vector2(110f, 2f);

        // {
        //     var btn = UnityEngine.Object.Instantiate(script.manager.configurableButtonPrefab, ui.transform, false);
        //     UnityEngine.Object.Destroy(btn.GetComponent<LayoutElement>());
        //     btn.GetComponent<UIDynamicButton>().label = "<";
        //     btn.GetComponent<UIDynamicButton>().button.onClick.AddListener(() =>
        //     {
        //         ui.popup.SetPreviousValue();
        //     });
        //     var btnRect = btn.GetComponent<RectTransform>();
        //     btnRect.anchoredPosition = new Vector2(0f, 0f);
        //     btnRect.sizeDelta = new Vector2(70f, -10f);
        //     btnRect.offsetMin += new Vector2(160f, 0f);
        //     btnRect.offsetMax += new Vector2(160f, 0f);
        //     btnRect.anchorMin = new Vector2(0f, 0f);
        //     btnRect.anchorMax = new Vector2(0f, 1f);
        // }

        // {
        //     var btn = UnityEngine.Object.Instantiate(script.manager.configurableButtonPrefab, ui.transform, false);
        //     UnityEngine.Object.Destroy(btn.GetComponent<LayoutElement>());
        //     btn.GetComponent<UIDynamicButton>().label = ">";
        //     btn.GetComponent<UIDynamicButton>().button.onClick.AddListener(() =>
        //     {
        //         ui.popup.SetNextValue();
        //     });
        //     var btnRect = btn.GetComponent<RectTransform>();
        //     btnRect.anchoredPosition = new Vector2(0f, 0f);
        //     btnRect.sizeDelta = new Vector2(70f, -10f);
        //     btnRect.offsetMin += new Vector2(-40f, 0f);
        //     btnRect.offsetMax += new Vector2(-40f, 0f);
        //     btnRect.anchorMin = new Vector2(1f, 0f);
        //     btnRect.anchorMax = new Vector2(1f, 1f);
        // }

        ui.popup.popupPanelHeight = popupPanelHeight;

        FinalizeBuild(ui);

        var rectTransform = ui.GetComponent<RectTransform>();

        if (location != null)
        {
            rectTransform.pivot = Vector2.zero;
            rectTransform.anchoredPosition = location;
        }
        if (size != null)
        {
            rectTransform.sizeDelta = size;
        }

        return ui;
    }
}
