using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class DropDownWithCreateNewBuilder : UIBuilder<DropDownWithCreateNewBuilder>
{
    public DropDownWithCreateNewBuilder(UIPart uIPart) : base(uIPart)
    {
    }

    public UIDynamicPopup Build(JSONStorableStringChooser jsc, UnityAction onCreateButton, float popupPanelHeight = 350f)
    {
        var prefab = script.manager.configurableScrollablePopupPrefab;

        var ui = Object.Instantiate(prefab).GetComponent<UIDynamicPopup>();

        ui.label = jsc.name;
        jsc.popup = ui.popup;

        script.RegisterStringChooser(jsc);

        Object.Destroy(ui.transform.Find("Slider").gameObject);
        // Object.Destroy(ui.transform.Find("Background").gameObject);

        RectTransform mainLabelTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Button/Text") as RectTransform;
        mainLabelTransform = Object.Instantiate(mainLabelTransform, ui.transform);
        mainLabelTransform.name = "Text";
        // labelTransform.anchorMax = new Vector2(0, 1);
        // labelTransform.anchorMin = new Vector2(0, 0);
        // mainLabelTransform.offsetMax = new Vector2(155, 0);
        mainLabelTransform.offsetMin = new Vector2(-400, 0);
        Text labelText = mainLabelTransform.GetComponent<Text>();
        labelText.text = jsc.name;
        labelText.color = Color.white;
        labelText.fontStyle = FontStyle.Bold;

        var popupLayout = ui.popup.GetComponent<LayoutElement>();
        popupLayout.minHeight = 50f;
        popupLayout.preferredHeight = 50f;

        var topButtonRect = ui.popup.topButton.GetComponent<RectTransform>();
        // topButtonRect.anchoredPosition = new Vector2(0f, 0f);
        topButtonRect.anchorMin = new Vector2(0.23f, 0.0f);
        topButtonRect.anchorMax = new Vector2(0.88f, 1.0f);
        topButtonRect.offsetMin = new Vector2(0, 0);
        topButtonRect.offsetMax = new Vector2(0, 0);

        var dropDownPanel = ui.transform.Find("PopupPanel").GetComponent<RectTransform>();
        dropDownPanel.anchoredPosition = new Vector2(-85f, 0f);
        // dropDownPanel.anchorMin = new Vector2(-0.5f, 0.5f);
        // dropDownPanel.anchorMax = new Vector2(1, 1);
        // dropDownPanel.offsetMin = new Vector2(-100f, 100f);
        dropDownPanel.offsetMax = new Vector2(0f, 0f);

        RectTransform buttonTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Button") as RectTransform;
        buttonTransform = Object.Instantiate(buttonTransform, ui.transform);
        buttonTransform.name = "CreateNewButton";
        buttonTransform.anchorMin = new Vector2(0.89f, 0.0f);
        buttonTransform.anchorMax = new Vector2(1.0f, 1.0f);
        buttonTransform.offsetMin = new Vector2(0, 0);
        buttonTransform.offsetMax = new Vector2(0, 0);
        // Image buttonImage = buttonTransform.GetComponent<Image>();
        // buttonImage.color = buttonColor;
        Button buttonOne = buttonTransform.GetComponent<Button>();
        buttonOne.onClick.AddListener(onCreateButton);
        Text labelOne = buttonTransform.Find("Text").GetComponent<Text>();
        labelOne.text = "+";

        ui.labelTextColor = Color.black;
        ui.label = "↓";
        var labelTransform = ui.transform.Find("Text").GetComponent<RectTransform>();
        labelTransform.anchoredPosition = new Vector2(370f, 2f);

        ui.popup.popupPanelHeight = popupPanelHeight;

        FinalizeBuild(ui);

        return ui;
    }
}
