using UnityEngine;
using UnityEngine.UI;

public abstract class PrefabBasedBuilder<T> : UIBuilder<T> where T : PrefabBasedBuilder<T>
{
    public static GameObject customPrefab;
    protected Vector2 location;
    protected Vector2 size;

    public PrefabBasedBuilder(UIPart uIPart) : base(uIPart)
    {
        if (customPrefab == null)
        {
            customPrefab = new GameObject("Custom UIDynamic");
            customPrefab.SetActive(false);
            RectTransform rt = customPrefab.AddComponent<RectTransform>();
            rt.anchorMax = new Vector2(0, 1);
            rt.anchorMin = new Vector2(0, 1);
            rt.offsetMax = new Vector2(535, -500);
            rt.offsetMin = new Vector2(10, -600);
            LayoutElement le = customPrefab.AddComponent<LayoutElement>();
            le.flexibleWidth = 1;
            le.minHeight = 50;
            le.minWidth = 350;
            le.preferredHeight = 50;
            le.preferredWidth = 500;

            CreatePrefab();
        }
    }

    public UIDynamic Build()
    {
        var customUIDynamic = BuildFromPrefab();

        FinalizeBuild(customUIDynamic);
        SetupLayout(customUIDynamic);

        return customUIDynamic;
    }

    protected void SetupLayout(UIDynamic customWidthButton)
    {
        LayoutElement le = customWidthButton.gameObject.AddComponent<LayoutElement>();
        var rectTransform = customWidthButton.GetComponent<RectTransform>();

        if (location != null)
        {
            rectTransform.pivot = Vector2.zero;
            rectTransform.anchoredPosition = location;
        }
        if (size != null)
        {
            rectTransform.sizeDelta = size;
        }
    }

    public T Location(Vector2 location)
    {
        this.location = location;
        return (T)this;
    }
    public T Size(Vector2 size)
    {
        this.size = size;
        return (T)this;
    }

    public abstract void CreatePrefab();

    public abstract UIDynamic BuildFromPrefab();
}
