using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class TextInputBuilder : UIBuilder<TextInputBuilder>
{
    private string label;
    private JSONStorableString storable;
    private int fontSize = 30;
    private bool fontBold = true;
    private Color labelColor = Color.white;
    private Vector2 location;
    private Vector2 size;

    public TextInputBuilder(MVRScript script, List<UIDynamic> uiElements, string label, JSONStorableString storable) : base(script, uiElements)
    {
        this.label = label;
        this.storable = storable;
    }

    public TextInputBuilder(UIPart uIPart, string label, JSONStorableString storable) : base(uIPart)
    {
        this.label = label;
        this.storable = storable;
    }

    public TextInputBuilder FontSize(int fontSize)
    {
        this.fontSize = fontSize;
        return this;
    }
    public TextInputBuilder FontBold()
    {
        fontBold = true;
        return this;
    }
    public TextInputBuilder LabelColor(Color labelColor)
    {
        this.labelColor = labelColor;
        return this;
    }
    public TextInputBuilder Location(Vector2 location)
    {
        this.location = location;
        return this;
    }
    public TextInputBuilder Size(Vector2 size)
    {
        this.size = size;
        return this;
    }
    public UIDynamicLabelInput Build()
    {
        var textInput = Utils.SetupTextInput(script, label, storable, rightSide, labelColor, fontBold);
        FinalizeBuild(textInput);
        var rectTransform = textInput.GetComponent<RectTransform>();
        if (location != null)
        {
            rectTransform.pivot = Vector2.zero;
            rectTransform.anchoredPosition = location;
        }
        if (size != null)
        {
            rectTransform.sizeDelta = size;
        }

        return textInput;
    }
}
