using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class ThreeWayButtonBuilder : PrefabBasedBuilder<ThreeWayButtonBuilder>
{
    private UnityAction callbackOne;
    private UnityAction callbackTwo;
    private UnityAction callbackThree;
    private Color color = UnityEngine.Color.white;
    private int groupBy;

    public ThreeWayButtonBuilder(UIPart uIPart, string label, UnityAction callbackOne, UnityAction callbackTwo, UnityAction callbackThree, int groupBy) : base(uIPart)
    {
        this.callbackOne = callbackOne;
        this.callbackTwo = callbackTwo;
        this.callbackThree = callbackThree;
        this.groupBy = groupBy;
    }
    public ThreeWayButtonBuilder Color(Color color)
    {
        this.color = color;
        return this;
    }
    public override void CreatePrefab()
    {
        RectTransform backgroundTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Background") as RectTransform;
        backgroundTransform = Object.Instantiate(backgroundTransform, customPrefab.transform);
        backgroundTransform.name = "Background";
        backgroundTransform.anchorMax = new Vector2(1, 1);
        backgroundTransform.anchorMin = new Vector2(0, 0);
        backgroundTransform.offsetMax = new Vector2(0, 0);
        backgroundTransform.offsetMin = new Vector2(0, -10);

        RectTransform labelTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Button/Text") as RectTransform; ;
        labelTransform = Object.Instantiate(labelTransform, customPrefab.transform);
        labelTransform.name = "Text";
        labelTransform.anchorMax = new Vector2(0, 1);
        labelTransform.anchorMin = new Vector2(0, 0);
        labelTransform.offsetMax = new Vector2(155, -5);
        labelTransform.offsetMin = new Vector2(5, 0);
        Text labelText = labelTransform.GetComponent<Text>();
        labelText.text = "Group By:";
        labelText.color = color;
        labelText.fontStyle = FontStyle.Bold;

        RectTransform buttonTransform = script.manager.configurableScrollablePopupPrefab.transform.Find("Button") as RectTransform;
        buttonTransform = Object.Instantiate(buttonTransform, customPrefab.transform);
        buttonTransform.name = "ButtonOne";
        buttonTransform.anchorMin = new Vector2(0.3f, 0.0f);
        buttonTransform.anchorMax = new Vector2(0.53f, 1.0f);
        buttonTransform.offsetMin = new Vector2(3, 0);
        buttonTransform.offsetMax = new Vector2(0, 0);
        // Image buttonImage = buttonTransform.GetComponent<Image>();
        // buttonImage.color = buttonColor;
        Button buttonOne = buttonTransform.GetComponent<Button>();
        Text labelOne = buttonTransform.Find("Text").GetComponent<Text>();

        buttonTransform = Object.Instantiate(buttonTransform, customPrefab.transform);
        buttonTransform.name = "ButtonTwo";
        buttonTransform.anchorMin = new Vector2(0.53f, 0.0f);
        buttonTransform.anchorMax = new Vector2(0.76f, 1.0f);
        buttonTransform.offsetMin = new Vector2(3, 0);
        buttonTransform.offsetMax = new Vector2(0, 0);
        Button buttonTwo = buttonTransform.GetComponent<Button>();
        Text labelTwo = buttonTransform.Find("Text").GetComponent<Text>();

        buttonTransform = Object.Instantiate(buttonTransform, customPrefab.transform);
        buttonTransform.name = "ButtonThree";
        buttonTransform.anchorMin = new Vector2(0.76f, 0.0f);
        buttonTransform.anchorMax = new Vector2(1.0f, 1.0f);
        buttonTransform.offsetMin = new Vector2(3, 0);
        buttonTransform.offsetMax = new Vector2(0, 0);
        Button buttonThree = buttonTransform.GetComponent<Button>();
        Text labelThree = buttonTransform.Find("Text").GetComponent<Text>();

        UIDynamicThreeWayButton uid = customPrefab.AddComponent<UIDynamicThreeWayButton>();
        uid.labelOne = labelOne;
        uid.labelTwo = labelTwo;
        uid.labelThree = labelThree;
        uid.buttonOne = buttonOne;
        uid.buttonTwo = buttonTwo;
        uid.buttonThree = buttonThree;
    }

    public override UIDynamic BuildFromPrefab()
    {
        {
            Transform t = Utils.ourCreateUIElement(customPrefab.transform, rightSide);
            UIDynamicThreeWayButton uid = t.GetComponent<UIDynamicThreeWayButton>();
            uid.labelOne.text = "Group";
            uid.labelTwo.text = "Layer";
            uid.labelThree.text = "Bundle";

            if (groupBy == GroupByType.Group)
            {
                uid.buttonOne.GetComponent<Image>().color = new Color(0.45f, 1f, 0.45f);
            }
            if (groupBy == GroupByType.Layer)
            {
                uid.buttonTwo.GetComponent<Image>().color = new Color(0.45f, 1f, 0.45f);
            }
            if (groupBy == GroupByType.Bundle)
            {
                uid.buttonThree.GetComponent<Image>().color = new Color(0.45f, 1f, 0.45f);
            }

            uid.buttonOne.onClick.AddListener(callbackOne);
            uid.buttonTwo.onClick.AddListener(callbackTwo);
            uid.buttonThree.onClick.AddListener(callbackThree);

            t.gameObject.SetActive(true);

            return uid;
        }
    }

    public class UIDynamicThreeWayButton : UIDynamicUtils
    {
        public Text labelOne;
        public Button buttonOne;
        public Text labelTwo;
        public Button buttonTwo;
        public Text labelThree;
        public Button buttonThree;
    }
}
