using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public class TwinButtonBuilder : UIBuilder<TwinButtonBuilder>
{
    private string leftLabel;
    private UnityAction leftCallback;
    private string rightLabel;
    private UnityAction rightCallback;
    private float firstWidth = 0.5f;
    private float firstStart = 0.0f;
    private bool labelBold = false;
    private Color buttonColor = Colors.BUTTON_DEFAULT;
    private Color textColor = Color.black;

    public TwinButtonBuilder(MVRScript script, List<UIDynamic> uiElements, string leftLabel, UnityAction leftCallback, string rightLabel, UnityAction rightCallback) : base(script, uiElements)
    {
        this.leftLabel = leftLabel;
        this.leftCallback = leftCallback;
        this.rightLabel = rightLabel;
        this.rightCallback = rightCallback;
    }
    public TwinButtonBuilder FirstWidth(float firstWidth)
    {
        this.firstWidth = firstWidth;
        return this;
    }
    public TwinButtonBuilder FirstStart(float firstStart)
    {
        this.firstStart = firstStart;
        return this;
    }
    public TwinButtonBuilder LabelBold(bool labelBold)
    {
        this.labelBold = labelBold;
        return this;
    }
    public TwinButtonBuilder ButtonColor(Color buttonColor)
    {
        this.buttonColor = buttonColor;
        return this;
    }
    public TwinButtonBuilder TextColor(Color textColor)
    {
        this.textColor = textColor;
        return this;
    }
    public UIDynamic Build()
    {
        var twinButton = Utils.SetupTwinButton(script, leftLabel, leftCallback, rightLabel, rightCallback, rightSide, buttonColor, textColor, firstWidth, firstStart, labelBold);
        FinalizeBuild(twinButton);

        return twinButton;
    }
}
