using System.Collections.Generic;
using UnityEngine;

public abstract class UIBuilder<T> where T : UIBuilder<T>
{
    protected MVRScript script;
    protected List<UIDynamic> uiElements;
    protected Transform parent;
    protected bool rightSide;
    protected bool constantElement;

    public UIBuilder(UIPart uIPart)
    {
        script = uIPart.script;
        uiElements = uIPart.uiElements;
    }

    public UIBuilder(MVRScript script, List<UIDynamic> uiElements)
    {
        this.script = script;
        this.uiElements = uiElements;
    }

    public T Parent(Transform parent)
    {
        this.parent = parent;
        return (T)this;
    }

    public T RightSide()
    {
        rightSide = true;
        return (T)this;
    }

    public T ConstantElement()
    {
        constantElement = true;
        return (T)this;
    }
    protected void FinalizeBuild(UIDynamic uiElement)
    {
        if (!constantElement)
        {
            uiElements.Add(uiElement);
        }
        if (parent != null)
        {
            uiElement.transform.SetParent(parent, false);
        }
    }
}
